{
  config,
  lib,
  pkgs,
  ...
}: let
  inherit (lib) types mkIf mkOption mkMerge;
in {
  options.my-config = {
    template.all.packages.enable = mkOption {
      description = "Enable common packages for all hosts";
      type = types.bool;
      default = true;
    };

    template.all.zfs.enable = mkOption {
      description = "Enable ZFS settings";
      type = types.bool;
      default = true;
    };

    template.any.nvidia.enable = mkOption {
      description = "Enable nvidia settings";
      type = types.bool;
      default = false;
    };

    template.any.system76.enable = mkOption {
      description = "Enable System76 settings";
      type = types.bool;
      default = false;
    };

    template.desktop.custom.enable = mkOption {
      description = "Enable customized GNOME desktop";
      type = types.bool;
      default = true;
    };

    template.server.services.enable = mkOption {
      description = "Enable server settings";
      type = types.bool;
      default = false;
    };
  };

  # See nixos-help > Option Definitions > Merging Configurations
  config =
    mkMerge
    [
      (mkIf config.my-config.template.desktop.custom.enable {
        services.xserver = {
          enable = true;
          desktopManager.gnome.enable = true;
          displayManager.gdm.enable = true;
        };

        hardware.pulseaudio.enable = false;
        security.rtkit.enable = true;
        services.pipewire = {
          enable = true;
          alsa.enable = true;
          alsa.support32Bit = true;
          pulse.enable = true;
        };

        environment.systemPackages = with pkgs; [
          amfora
          catppuccin-cursors.mochaRed
          dconf2nix
          emacs
          firefox
          gnome.dconf-editor
          gnome.gnome-screenshot
          gnome.gnome-terminal
          gnome.gnome-tweaks
          gnomeExtensions.appindicator
          gnomeExtensions.openweather
          gnomeExtensions.pano
          gnomeExtensions.quake-mode
          gnomeExtensions.unite
          gnomeExtensions.uptime-indicator
          gnomeExtensions.user-themes
          lagrange
          mullvad-vpn
          nextcloud-client
          rofi
          rofimoji
          zellij
        ];

        fonts.fonts = with pkgs; [
          font-awesome
          nerdfonts
          noto-fonts
          noto-fonts-extra
          rubik
        ];

        fonts.fontconfig = {
          antialias = true;
          hinting = {
            enable = true;
            style = "slight";
            autohint = false;
          };
          subpixel = {
            lcdfilter = "default";
            rgba = "rgb";
          };
          defaultFonts = {
            serif = ["Noto Serif"];
            sansSerif = ["Noto Sans"];
            monospace = ["JetBrainsMono Nerd Font Mono"];
          };
        };

        programs.gnupg = {
          agent.enable = true;
          agent.pinentryFlavor = "gnome3";
          dirmngr.enable = true;
        };

        services.mullvad-vpn.enable = true;
      })

      (mkIf config.my-config.template.any.system76.enable {
        services.system76-scheduler.enable = true;
        hardware.system76.power-daemon.enable = true;
      })

      (mkIf config.my-config.template.any.nvidia.enable {
        nixpkgs.config.allowUnfree = true;
        services.xserver.videoDrivers = ["nvidia"];
        hardware = {
          opengl.enable = true;
          nvidia.package = config.boot.kernelPackages.nvidiaPackages.stable;
          nvidia.modesetting.enable = true;
        };
      })

      (mkIf config.my-config.template.all.zfs.enable {
        services.zfs.autoScrub.enable = true;
        services.sanoid = {
          enable = true;
          datasets.rpool.hourly = 24;
          datasets.rpool.daily = 7;
          datasets.rpool.weekly = 4;
          datasets.rpool.monthly = 12;
          datasets.rpool.recursive = true;
        };
      })

      (mkIf config.my-config.template.all.packages.enable {
        environment.systemPackages = with pkgs; [
          alejandra
          autossh
          bat-extras.batdiff
          bat-extras.batgrep
          bat-extras.batman
          bat-extras.batwatch
          bat-extras.prettybat
          btop
          charm
          coreutils
          deadnix
          diff-so-fancy
          du-dust
          duf
          exa
          fd
          gcc
          git
          glow
          gnumake
          go
          go-tools
          gopls
          gops
          gosec
          gum
          httpie
          hugo
          hyperfine
          jq
          killall
          ldns
          libnotify
          lnav
          lsof
          neofetch
          nix-diff
          nix-doc
          nix-du
          nix-tree
          nmap
          node2nix
          nodejs
          pass
          pinentry
          prettyping
          python3Full
          ranger
          ripgrep
          rustup
          shellcheck
          shfmt
          skate
          starship
          statix
          stow
          tealdeer
          trash-cli
          unzip
          vim
          w3m
          wget
          xorg.xauth
          xclip
          xsel
          yarn
          zip
          zsh-nix-shell
        ];
      })

      (mkIf config.my-config.template.server.services.enable {
        programs.atop = {
          enable = true;
          atopacctService.enable = true;
          atopService.enable = true;
          atopRotateTimer.enable = true;
        };

        services.clamav = {
          updater.enable = true;
          daemon.enable = true;
        };

        services.rsyslogd = {
          enable = true;
          extraConfig = "*.* @10.0.0.27:514";
        };

        services.tuptime = {
          enable = true;
          timer.enable = true;
        };
      })
    ];
}
